// BASED OF HOPEFRF SAMPLE CODE & http://dlehard.narod.ru/quick_start.pdf

#define F_CPU 8000000UL
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>

/* RFM12B INTERFACE */
#define SCK 7 // SPI clock
#define MISO 6 // SPI Data input (RFM12B side)
#define MOSI 5 // SPI Data output (RFM12B side)
#define CS 3 // SPI SS (chip select) (PORTC)
#define NIRQ 4 // (PORTC)
/* IO CONTROL */
#define HI(x) PORTB |= (1<<(x))
#define LO(x) PORTB &= ~(1<<(x))
#define CS_HI() PORTC |= (1<<CS)
#define CS_LO() PORTC &= ~(1<<CS)
#define NIRQ_HI() PORTC |= (1<<NIRQ)
#define NIRQ_LO() PORTC &= ~(1<<NIRQ)
#define WAIT_NIRQ_LOW() while(PINC&(1<<NIRQ))

/* LED */
#define LED 1
#define LED_OFF() PORTC &= ~(1<<LED)
#define LED_ON() PORTC |= (1<<LED)

union {
	float f;
	unsigned char b[4];
} u;

unsigned volatile int Ain;  //ADC value
unsigned volatile int avgADC[10]; //Moving average of ADC values
unsigned volatile char avgCount; //Index of moving average
unsigned volatile long sum; //Total of 1000 samples
unsigned volatile int adcCount; //Number of samples
unsigned volatile char newADCFlag; 
unsigned int avgSum;


char SPI_TX_DONE_FLAG;

void portInit() {
	DDRB = (1<<MOSI) | (1<<SCK) | (1<<4);
	DDRC = (1<<LED) | (1<<CS);
	PORTB |= (1<<MISO); //Set MISO and SS to pulled up
	CS_HI();
}


void SPI_MasterTransmit(char cData)
{
	/* Start transmission */
	SPDR = cData;
	/* Wait for transmission complete */
	while
	(!(SPSR & (1<<SPIF)));
}

//Send command to wireless transceiver
void writeCmd(unsigned int cmd) {
	CS_LO();
	SPI_MasterTransmit((cmd >> 8));
	SPI_MasterTransmit(cmd);
	CS_HI();
}

void spiInit() {
	SPCR = (1<<SPE) | (1<<MSTR);
	// no interupt | enable SPI | Master | CPOL=0 | CPHA=0 | Fosc/4 |	
}

//Set up the rf module
void rfInit() {
	writeCmd(0x80F7); //EL,EF,915band,12.0pF
	writeCmd(0x8239); //!er,!ebb,ET,ES,EX,!eb,!ew,DC
	writeCmd(0xA640); //frequency select
	writeCmd(0xC647); //4.8kbps
	writeCmd(0x94A0); //VDI,FAST,134kHz,0dBm,-103dBm
	writeCmd(0xC2AC); //AL,!ml,DIG,DQD4
	writeCmd(0xCA81); //FIFO8,SYNC,!ff,DR
	writeCmd(0xCED4); //SYNC=2DD4,AG
	writeCmd(0xC483); //@PWR,NO RSTRIC,!st,!fi,OE,EN
	writeCmd(0x9820); //!mp,45kHz,MAX OUT
	writeCmd(0xCCA7); //OB1,ACOB0, LPX,Iddy,CDDIT,CBW0
	writeCmd(0xE000); //NOT USED
	writeCmd(0xC800); //NOT USED
	writeCmd(0xC040); //1.66MHz,2.2V
}

//Send a byte of wireless data
void rfSend(unsigned char data){

	LED_ON();
 	WAIT_NIRQ_LOW();
	writeCmd(0xB800 + data);
	LED_OFF();
}

//Send a float over wireless
void rfSendFloat(){
	
	for (char i=0; i<4; i++)
	{
	 	WAIT_NIRQ_LOW();
		writeCmd(0xB800 + u.b[i]);
	}
}


void timerInit()
{
  //set up timer 0 for 1 mSec timebase 
  TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
  OCR0A = 124;  		//set the compare reg to 250 time ticks
  //set prescalar to divide by 64 
  TCCR0B= 3; 	
  // turn on clear-on-match
  TCCR0A= (1<<WGM01) ;
}

//**********************************************************
//timer 0 compare ISR used to poll adc
ISR (TIMER0_COMPA_vect)
{
	//LED_ON();
	//read ADC 
	if (adcCount < 256)
	{   
		Ain = ADCL;
		Ain |= ((0x03&ADCH)<<8);
	
		if (Ain > 0) sum  += Ain;
		adcCount++;
	} else 
	{	
		adcCount = 0;
		if (avgCount == 9) avgCount = 0;
		else avgCount++;
		avgADC[avgCount] = (sum >> 8);
		sum = 0;
		newADCFlag = 1;
	}

	//Get new ADC value
  ADCSRA |= 1<<ADSC;
  //LED_OFF();


}

int main() {
	SPI_TX_DONE_FLAG = 1;
	asm("cli");
	portInit();
	spiInit();
	rfInit();
	timerInit();

	//Initialize ADC
	ADMUX = (1<<REFS0);
	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7;
	//LED_ON();
	sei();
	u.f = 1.5;	
	while(1){
		//If we have a new value to send
		if (newADCFlag)
		{
			avgSum = 0;
			for (int i=0; i < 10; i++)
			{
				avgSum += avgADC[i];
			}
			u.f = ((avgSum/10.0)-658)*15.0; //Convert to inch pounds of torque
			if (u.f < 0) u.f = 0.0;
			LED_ON();
			writeCmd(0x0000);
			rfSend(0x2D); // SYNC
			rfSend(0xD4); // SYNC
			rfSendFloat();
			LED_OFF();
			newADCFlag=0;
		}

	}
}
